ResourceFetchError

Struct ResourceFetchError 

pub struct ResourceFetchError {
    pub resource_name_short: String,
    pub resource_name_full: String,
}
Indicates a resource that did not exist in Resources.

Fields

resource_name_short: String

Short type name of the CmdBlock::Input type.

resource_name_full: String

Full type name of the CmdBlock::Input type.

Implementations

Source

impl ResourceFetchError

pub fn new<R>() -> Self

Returns a new ResourceFetchError with the given type’s name.

pub fn resource_name_short(&self) -> &str

Returns the short type name, e.g. String

For generic types, this returns the short type names for the type and the parameter, e.g. Option<String>.

pub fn resource_name_full(&self) -> &str

Returns the full type name, e.g. std::string::String

For generic types, this returns the short type names for the type and the parameter, e.g. std::option::Option<std::string::String>.