pub struct ResourceFetchError {
pub resource_name_short: String,
pub resource_name_full: String,
}Expand description
Indicates a resource that did not exist in Resources.
Fields
resource_name_short: StringShort type name of the CmdBlock::Input type.
resource_name_full: StringFull type name of the CmdBlock::Input type.
Implementations
Sourceimpl ResourceFetchError
impl ResourceFetchError
Sourcepub fn resource_name_short(&self) -> &str
pub fn resource_name_short(&self) -> &str
Returns the short type name, e.g. String
For generic types, this returns the short type names for the type and
the parameter, e.g. Option<String>.
Sourcepub fn resource_name_full(&self) -> &str
pub fn resource_name_full(&self) -> &str
Returns the full type name, e.g. std::string::String
For generic types, this returns the short type names for the type and
the parameter, e.g. std::option::Option<std::string::String>.